/* 
 * main.c 
 * main routine of ka9q-remote
 * Harald Milz (hm@seneca.ix.de) 1995
 * This file is copyrighted according to the 
 * GNU Public License 2.0 or later. 
 * taken and adapted from: 
 */
 
/* Main-level NOS program:
 *  initialization
 *  keyboard processing
 *  generic user commands
 *
 * Copyright 1991 Phil Karn, KA9Q
 */

#include <stdio.h>
#include <time.h>
#include <sys/socket.h>
#include <errno.h>
#include <netdb.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <unistd.h>

/* from ka9q/remote.h */
#define SYS_RESET       1
#define SYS_EXIT        2
#define KICK_ME         3

/* from ka9q/socket.h */
#define IPPORT_REMOTE   1234

/* from ka9q/global.h: */
#define        NULLCHAR (char *)0      /* Null character pointer */
extern char *sys_errlist[];

#define int16           unsigned short
#define int32           unsigned int


/* Generate system command packet. Synopsis:
 * ka9q-remote [-v] [-n] [-p port#] [-k key] [-a hostname] -c {reset|exit|kickme} <hostname> 
 */
int 
main (argc,argv,p)
int argc;
char *argv[];
void *p;
{
  struct sockaddr_in fsock;
  struct sockaddr_in sin;
  int s,c;
  char *data,x;
  int16 port,len;
  char *key = NULLCHAR;
  int klen;
  int32 addr = 0;
  char *cmd,*host;
  int verbose = 0;      /* speak up a little bit louder ... */
  int really = 1;	/* are we going to do it? */
  
  if (argc < 2) {
    puts ("ka9q-remote [-v] [-n] [-p port#] [-k key] [-a hostname] -c {reset|exit|kickme} <hostname>\n");
    exit (-1);
  }
  port = IPPORT_REMOTE;	/* Set default */
  optind = 1;		/* reinit getopt() */
  while((c = getopt(argc,argv,"a:c:p:k:vn")) != EOF){
    switch(c){
    case 'a':
      resolve (optarg, &sin);
      addr = sin.sin_addr.s_addr;
      if (verbose)
        printf ("opt a: %s %s\n", optarg, show (addr));
      break;
    case 'p':
      port = atoi(optarg);
      break;
    case 'k':
      key = optarg;
      klen = strlen(key);
      break;
    case 'c':
      cmd = optarg;
      break;
    case 'v':
      verbose = 1;
      break;
    case 'n':
      really = 0;
      break;
    }
  }
  if(optind > argc - 1){
    printf("Insufficient args\n");
    return -1;
  }
  host = argv[optind++];
  if((s = socket(AF_INET,SOCK_DGRAM,0)) == -1){
    printf("socket failed\n");
    return 1;
  }
  len = 1;
  /* Did the user include a password or kickme target? */
  if(addr != 0)
    len += sizeof(int32);
  
  if(key != NULLCHAR)
    len += klen;
  
  if(len == 1)
    data = &x;
  else
    data = (char *)malloc(len); /* mallocw not needed for Unix */
  
  fsock.sin_family = AF_INET;
  resolve (host, &sin);
  fsock.sin_addr.s_addr = sin.sin_addr.s_addr;
  /* seems byte swapping is needed here :-( */
  fsock.sin_port = (((port & 0xFF) << 8) | ((port & 0xFF00) >> 8));
  switch(cmd[0]){
  case 'r':
    data[0] = SYS_RESET;
    if(key != NULLCHAR)
      strncpy(&data[1],key,klen);
    break;
  case 'e':
    data[0] = SYS_EXIT;
    if(key != NULLCHAR)
      strncpy(&data[1],key,klen);
    break;
  case 'k':
    data[0] = KICK_ME;
    if(addr != 0)
      data[1] = addr;
    break;
  default:
    printf("Unknown command %s\n",cmd);
    goto cleanup;
  }
  /* Form the command packet and send it */
  if (verbose) {
    printf ("sending packet\n");
    printf ("host:\t%s %s\n", host, show(sin.sin_addr.s_addr));
    printf ("port:\t%d\n", port);
    printf ("cmd:\t%s (%d)\n", cmd, data[0]);
    printf ("data:\t%X\n", data[1]);
  }  
  if (really) {
    if(sendto(s,data,len,0,(const struct sockaddr *)&fsock,sizeof(fsock)) == -1){
      printf("sendto failed: %s\n",sys_errlist[errno]);
      goto cleanup;
    }
  }
 cleanup:
  if(data != &x)
    free(data);
  close(s);
  return 0;
}
